// Libraries import
#include <Wire.h>
#include <Adafruit_PWMServoDriver.h>

Adafruit_PWMServoDriver pwm = Adafruit_PWMServoDriver();

// Variables definition
int ciclo = 0; //Thermal cycle (0-warming, 1-cooling)
int Enable = 9; //TEC power driver PIN
int angle_X = -45; //Initial titling angle (X axis)
int angle_Y = -45; //Initial titling angle (Y axis)
int pulse_w_X; //PWM servo-motor X axis
int pulse_w_Y; //PWM servo-motor Y axis
int X_cycle = 0; //Axis of inclination (0-X, 1-Y)
int PWM; //PWM TEC power driver
int i = 0; //Time variable
int anglesec = 0; //Interrupt for Y axis tilt beginning 
int timestep = 10000; //Number of cycles for a single temperature step (10000 cycles = 60 seconds)
int readEnable = 12; //Interrupt for external sensor read beginning 
int readDisable = 13; //Interrupt for external sensor read ending 
int Anglezero = 8; //Interrupt for initial condition setup
int Changeangle = 11; //Interrupt for tilt step 
int enableInterrupt = 0; //Interrupt function
int sensorValue; //Raw ADC internal temperature 
int sensorValue1; //Raw ADC external temperature

float Kp = 4000; //PID gain constant Kp
float Ki = 0.05; //PID gain constant Ki
float error; //Temperature error
float Output; //PWM value
float ITerm; //Error estimated on the basis of Kp 
float setpoint = -10; //Testing temperature range: low 
float endpoint = 46; //Testing temperature range: high
float prevTempint = 0.0; //Initial averaged internal temperature
float prevTempext = 0.0; //Initial averaged external temperature 
float prevresistor = 0.0; //Average internal thermistor value
float prevresistor1 = 0.0; //Average external thermistor value
float thermistornominal = 100000.0; //Nominal resistance of the thermistor at 25 °C
float A = 2.295059308e-3; //Steinhart–Hart coefficient
float B = 0.04687545506e-4; //Steinhart–Hart coefficient
float C = 6.623402025e-7; //Steinhart–Hart coefficient
float volt; //Voltage at internal resistor
float volt1; //Voltage at external resistor
float resistor; //Resistance of the internal thermistor
float resistor1; //Resistance of the external thermistor
float mediaresistor; //Average resistance of the internal thermistor
float mediaresistor1; //Average resistance of the external thermistor
float tempint; //Internal temperature
float tempext; //External temperature
float mediatempint; //Averaged internal temperature
float mediatempext; //Averaged external temperature

//Setup function
void setup ()
{

Serial.begin (115200);

//Setting Pin mode
pinMode(Enable, OUTPUT);
pinMode(readEnable, OUTPUT); 
pinMode(readDisable, OUTPUT); 
pinMode(Changeangle, OUTPUT);
pinMode(Anglezero, OUTPUT);

digitalWrite(Enable, HIGH);

// inizialize PCA9685 chip 
pwm.begin();  
pwm.setPWMFreq(60);  // Analog servos update at ~ 60 Hz 

delay(10);

digitalWrite(readEnable, LOW);
digitalWrite(readDisable, LOW);
digitalWrite(Changeangle, LOW);
digitalWrite(Anglezero, LOW);
}

void setServoPulse(uint8_t n, double pulse) 
{
  double pulselength;
  
  pulselength = 1000000;   
  pulselength /= 60;   
  Serial.print(pulselength); 
  pulselength /= 4096;  
  Serial.print(pulselength); 
  pulse *= 1000000;  
  pulse /= pulselength;
  pwm.setPWM(n, 0, pulse);
}

//Loop function
void loop ()
{
   sensorValue = analogRead(A0); //Read analog value of internal resistor
   volt = sensorValue*0.0048828125; //Convert raw analog data to Volts
   resistor = ((5*98300)/volt)-98300; //Estimation of internal thermistor resistance
   mediaresistor = (resistor+prevresistor)/2; 
   prevresistor = resistor;
   tempint = (1/(A + B*(log(mediaresistor)) + C*(log(mediaresistor)*log(mediaresistor)*log(mediaresistor))))-273.15; //Steinhart–Hart equation for internal teperature estimation
   
   sensorValue1 = analogRead(A1); //Read analog value of external resistor
   volt1 = sensorValue1*0.0048828125; //Convert raw analog data to Volts
   resistor1 = ((5*98300)/volt1)-98300; //Estimation of external thermistor resistance
   mediaresistor1 = (resistor1+prevresistor1)/2;
   prevresistor1 = resistor1;
   tempext = (1/(A + B*(log(mediaresistor1)) + C*(log(mediaresistor1)*log(mediaresistor1)*log(mediaresistor1))))-273.15; //Steinhart–Hart equation for external teperature estimation
   
   mediatempint = (tempint + prevTempint)/2;
   prevTempint = tempint;
   mediatempext = (tempext + prevTempext)/2; 
   prevTempext = tempext;

// X axis tilting cycle
  if (X_cycle == 0) 
  {
    angle_Y = 0; //Set Y angle to 0 degrees
    pulse_w_X  = map (constrain(angle_X, -45, 45), 0, 90, 350, 178);
    pulse_w_Y  = map (constrain(angle_Y, -45, 45), 0, 45, 329, 239);
    
    if (enableInterrupt == 0)
      {
        digitalWrite(readEnable, HIGH); //Start read sensor (interrupt)
        delay(100);
        digitalWrite(readEnable, LOW);
        
        enableInterrupt = 1;
        
        digitalWrite(Anglezero, HIGH); //Set start angle (interrupt)
        delay(100);
        digitalWrite(Anglezero, LOW);
      }
    pwm.setPWM (15, 0, pulse_w_X);
    pwm.setPWM (14, 0, pulse_w_Y);

//Warming cycle begin
    if(ciclo == 0) 
      {

        if (setpoint <= (tempext))
          error =(mediatempint)-setpoint; 
        if (setpoint >= (mediatempext))
          error = setpoint - (mediatempint);
          ITerm += (Ki*error);
        if (ITerm > 4095) 
          ITerm = 4095;
        if (ITerm < 0)
          ITerm = 4095;
          Output = Kp * error + ITerm ; 
        if (Output > 4095) 
          Output = 4095;
        if (Output < 0)
          Output = 0;
        PWM = int(Output);
        if ((int (tempext)) >= setpoint)
          {
          pwm.setPWM (13, 0,PWM);
          pwm.setPWM (12, 0, 0); 
          }
        else 
          {
          pwm.setPWM (13, 0, 0);
          pwm.setPWM (12, 0, PWM/2); 
          } 
        if(setpoint >= endpoint)
        ciclo = 1;
        i++;
        if (i > timestep)
          {
          i = 0;
          setpoint = setpoint + 2; //Increment internal temperature of one step 
          }
     }

//Cooling cycle begin
     if(ciclo == 1) 
      { 
        if (endpoint < (tempext))
          error = (mediatempint) - endpoint; 
        if (endpoint > (mediatempext))
          error = endpoint - (mediatempint);
          ITerm += (Ki*error);
        if (ITerm > 4095)
          ITerm = 4095;
        if (ITerm < 0)
          ITerm = 4095;
          Output = Kp * error +ITerm ;
        if (Output > 4095)
          Output = 4095;
        if (Output < 0)
          Output = 0;
        PWM = int(Output);
        if ((int (tempext)) < endpoint)
          {
          pwm.setPWM (13, 0, 0);
          pwm.setPWM (12, 0, PWM/2); 
          }
        else 
          {
           pwm.setPWM (13, 0, PWM);
           pwm.setPWM (12, 0, 0);   
          } 
        if(setpoint >= endpoint)
          ciclo = 1;
          i++;
        if (i > timestep)
          {  
          i = 0;
          endpoint = endpoint - 2; //Decrement internal temperature of one step 
          } 
       }
       
     if(endpoint <= -12) 
      {
      pwm.setPWM (13, 0, 0);
      pwm.setPWM (12, 0, 0);
      angle_X = angle_X + 5; //Increment tilt angle of 5 degrees
      
      digitalWrite(Changeangle, HIGH); //Interrupt: angular step completed
      delay(100);
      digitalWrite(Changeangle, LOW);                       
      delay(timestep); 
      
      setpoint = -12;
      endpoint = 46;
      ciclo = 0;
      if(angle_X == 50)
        {
        X_cycle = 1;
        angle_Y = -45;
        setpoint = -10;
        endpoint = 46;
        ciclo = 0; 
        }
      }
}

//Y axis tilting cycle
if(X_cycle == 1) 
{
 angle_X = 0; //Set X angle to 0 degrees
 pulse_w_X = map(constrain(angle_X, -45, 45), 0, 90, 350, 178);
 pulse_w_Y = map(constrain(angle_Y, -45, 45), 0, 45, 329, 239);
 pwm.setPWM (15, 0, pulse_w_X);
 pwm.setPWM (14, 0, pulse_w_Y);
 
 if(anglesec == 0)
  {
  digitalWrite(Anglezero, HIGH); //Y axis tilting started (interrupt)
  delay(100);
  digitalWrite(Anglezero, LOW);
  anglesec = 1;
  }

 //Warming cycle begin
 if(ciclo == 0) 
  {
  if (setpoint <= (tem-pext))
    error = (mediatempint) - setpoint; 
  if (setpoint >= (mediatempext))
    error = setpoint - (mediatempint);
    ITerm += (Ki*error);
  if (ITerm > 4095)
    ITerm = 4095;
  if (ITerm < 0)
    ITerm = 4095;
  Output = Kp * error + ITerm;
  if (Output > 4095)
    Output = 4095;
  if (Output < 0)
    Output = 0;
  PWM = int(Output);
  if ((int(tempext)) >= setpoint)
   {
   pwm.setPWM (13, 0, PWM);
   pwm.setPWM (12, 0, 0); 
   }
  else 
   {
   pwm.setPWM (13, 0, 0);
   pwm.setPWM (12, 0, PWM/2); 
   } 
  if(setpoint >= endpoint)
    ciclo = 1;
    i++;
  if (i > timestep)
    { 
    i = 0;
    setpoint = setpoint + 2; //Increment internal temperature of one step 
    }
  }

//Cooling cyle begin
if(ciclo == 1) 
 { 
  if (endpoint < (tempext))
    error = (mediatempint) - endpoint; 
  if (endpoint > (mediatempext))
    error = endpoint - (mediatempint);
    ITerm += (Ki*error);
  if (ITerm > 4095)
    ITerm = 4095;
  if (ITerm < 0)
    ITerm = 4095;
    Output = Kp * error + ITerm;
  if (Output > 4095)
    Output = 4095;
  if (Output < 0)
    Output = 0;
  PWM = int(Output);
  if ((int(tempext)) < endpoint)
    {
    pwm.setPWM (13, 0, 0);
    pwm.setPWM (12, 0, PWM/2);  
    }
  else 
    {
    pwm.setPWM (13,0,PWM);
    pwm.setPWM (12,0,0);  
    } 
 if(setpoint >= endpoint)
  ciclo = 1;
  i++;
 if (i > timestep)
  { 
  i = 0;
  endpoint = endpoint - 2; //Decrement internal temperature of one step 
  }  
 }

if(endpoint <= -12)
 {
 pwm.setPWM (13, 0,0);
 pwm.setPWM (12, 0, 0);
  
 delay(timestep); 
 angle_Y = angle_Y + 5; //Increment tilt angle of 5 degrees
 
 digitalWrite(Changeangle, HIGH); //Interrupt: angular step completed
 delay(100);
 digitalWrite(Changeangle, LOW);
   setpoint = -10;
   endpoint = 46;
   ciclo = 0;
 if(angle_Y == 50) 
  {
  X_cycle = 0;
  setpoint = -14;
  endpoint = 46;
  ciclo = 0;
  angle_Y = 0;
  angle_X = -45;
  digitalWrite (readDisable, HIGH);
  delay (100);
  digitalWrite (readDisable, LOW);
  enableInterrupt = 0;
  delay (57600000); //Cell off for 8h
  }
 } 
}

//Seral print data 
Serial.print(mediatempint); //Internal temperature
Serial.print(" , ");
Serial.print(mediatempext); //External temperature
Serial.print(" , ");
Serial.print(i); //Current cycle temperature step  
Serial.print(" , ");
Serial.print(setpoint); //Current temperature value
Serial.print(" , ");
Serial.print(endpoint); //Temperature to be reached
Serial.print(" , ");
Serial.println(PWM); // Current PWM value (power of TEC drive)
                            
}
