// Libraries import
#include <Wire.h>
#include <Adafruit_LSM9DS0.h>
#include <Adafruit_Sensor.h>

Adafruit_LSM9DS0 lsm = Adafruit_LSM9DS0();

// Variables definition
int enableRead = 1; //Sensor readability
int temp; //Sensor temperature
int X_AXIS_A; //X axis acceleration 
int Y_AXIS_A; //Y axis acceleration 
int Z_AXIS_A; //Z axis acceleration 
int angle; //Titing angle

const byte interruptPinEna = 0; //Interrupt for sensor read beginning 
const byte interruptPinDis = 1; //Interrupt for sensor read ending 
const byte ChangeanglePin = 4; //Interrupt for cell inclination variation
const byte Setangle = 6; //Interrupt for tilting angle reset


void setupSensor()
  {
  lsm.setupAccel(lsm.LSM9DS0_ACCELRANGE_2G); //Inizialize sensor and set acceleration range to 2g
  }

//Setup function
void setup() 
  {
  #ifndef ESP8266
  while (!Serial);     
  #endif
  Serial.begin(115200); 
  
  //Pin mode setting
  pinMode (interruptPinEna, INPUT);
  pinMode (interruptPinDis, INPUT);
  pinMode (ChangeanglePin, INPUT);
  pinMode (Setangle, INPUT);
  
  //Setting interrupt functions and trigger mode
  attachInterrupt (interruptPinEna,readEnable, RISING); 
  attachInterrupt (interruptPinDis,readDisable, RISING);
  attachInterrupt (ChangeanglePin,Changeangle, RISING); 
  attachInterrupt (Setangle,SetAngle, RISING); 
  if (!lsm.begin())
    {
    while (1);
    }
  }

//Loop function
void loop() 
  {
  lsm.read();
  if (enableRead == 0) //Start sensor reading
    {
    Serial.print( 21.00 +(((float)lsm.temperature)/8)); //Print sensor temperature
    Serial.print(", "); 
    Serial.print((int)lsm.accelData.x); //Print X axis acceleration
    Serial.print(", ");
    Serial.print((int)lsm.accelData.y); //Print Y axis acceleration     
    Serial.print(", "); 
    Serial.print((int)lsm.accelData.z); //Print Z axis acceleration
    Serial.print(", ");
    Serial.println(angle); //Print tilting angle  
    delay(500);
    }
  }

//Interrupt functions
void readEnable() //Start sensor read
  {
  enableRead = 0; 
  }

void readDisable() //Stop sensor read
  {
  enableRead = 1;
  }

void Changeangle() //Tilt angle change
  {
  angle = angle + 5; //Increment tilting value
  }

void SetAngle() //Set to intial titlting angle  
  {
  angle = - 45; 
  }
